
-- Collation rules

-- Indexes can only be created on string columns if they use a BIN2 collation. The LastName variable uses BIN2 collation. 
-- FirstName uses the database default, which is CI_AS (case-insensitive, accent-sensitive).

ALTER DATABASE Northwind COLLATE Latin1_General_100_CI_AS
GO
--
CREATE TABLE Employees (
EmployeeID int NOT NULL , 
LastName nvarchar(20) COLLATE Latin1_General_100_BIN2 NOT NULL INDEX IX_LastName HASH(LastName) WITH (BUCKET_COUNT=1024), 
FirstName nvarchar(10) NOT NULL ,
CONSTRAINT PK_Employees PRIMARY KEY NONCLUSTERED HASH(EmployeeID)  WITH (BUCKET_COUNT=1024)
) WITH (MEMORY_OPTIMIZED=ON, DURABILITY=SCHEMA_AND_DATA)
GO

-- all supported collations for (var)char columns in memory-optimized tables
select * from sys.fn_helpcollations()
where collationproperty(name, 'codepage') = 1252;

-- all supported collations for indexes on memory-optimized tables and 
-- comparison/sorting in natively compiled stored procedures
select * from sys.fn_helpcollations() where name like '%BIN2'

/*
Truncation of UTF-16 data is not supported inside natively compiled stored procedures. 
This means that n(var)char(n) values cannot be converted to type n(var)char(i), 
if i < n, if the collation has _SC property. For example, the following is not supported:
*/
-- column definition using an _SC collation
 c2 nvarchar(200) collate Latin1_General_100_CS_AS_SC not null 
-- assignment to a smaller variable, requiring truncation
 declare @c2 nvarchar(100) = '';
 select @c2 = c2

 /*
 The following example shows some of the implications and workarounds for the collation limitations in In-Memory OLTP. 
 The example uses the Employees table specified above. This sample list all employees . 
 Notice that for LastName, due to the binary collation, the upper case names are sorted before lower case. 
 Therefore, 'Thomas' comes before 'nolan' because upper case characters have lower code points. 
 FirstName has a case-insensitive collation. So, sorting is by letter of the alphabet, not code point of the characters.
 */

 -- insert a number of values
INSERT Employees VALUES (1,'thomas', 'john')
INSERT Employees VALUES (2,'Thomas', 'rupert')
INSERT Employees VALUES (3,'Thomas', 'Jack')
INSERT Employees VALUES (4,'Thomas', 'annie')
INSERT Employees VALUES (5,'nolan', 'John')
GO

-- =====================================================
SELECT EmployeeID, LastName, FirstName FROM Employees
ORDER BY LastName, FirstName
GO

-- =====================================================
-- specify collation: sorting uses case-insensitive collation, thus 'nolan' comes before 'Thomas'
--
SELECT * FROM Employees
ORDER BY LastName COLLATE Latin1_General_100_CI_AS, FirstName
GO

-- =====================================================
-- retrieve employee by Name
-- must use BIN2 collation for comparison in natively compiled stored procedures
--
CREATE PROCEDURE usp_EmployeeByName @LastName nvarchar(20), @FirstName nvarchar(10)
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER
AS BEGIN ATOMIC WITH 
(TRANSACTION ISOLATION LEVEL = SNAPSHOT,
LANGUAGE = N'us_english')

  SELECT EmployeeID, LastName, FirstName FROM dbo.Employees
  WHERE 
    LastName = @LastName AND
    FirstName COLLATE Latin1_General_100_BIN2 = @FirstName

END
GO

-- this does not return any rows, as EmployeeID 1 has first name 'john', which is not equal to 'John' in a binary collation
EXEC usp_EmployeeByName 'thomas', 'John'

-- this retrieves EmployeeID 1
EXEC usp_EmployeeByName 'thomas', 'john'
