use ContosoRetailDW
go

-- Data Allocations
SELECT index_id, rows, data_compression_desc, type_desc, total_pages
FROM sys.partitions p
JOIN sys.allocation_units au
ON p.partition_id = au.container_id
WHERE OBJECT_NAME(object_id) = 'FactOnlineSales';
GO

-- sys.system_internals_partitions keeps track of columnstore segments
select t.name, ip.* 
from sys.system_internals_partitions ip
join sys.tables t
 on ip.object_id = t.object_id
join sys.column_store_segments s
 on ip.partition_id = s.partition_id

select * from sys.column_store_segments

-- Basic Columnstore
/* Encoding type column */
select * from sys.column_store_segments
select * from sys.column_store_row_groups 
/* Type column
1  Hash dictionary containing int values
3  Hash dictionary containing string values
4  Hash dictionary containing float values
*/
select * from sys.column_store_dictionaries
order by column_id

-- vs select # of distinct values
select count (distinct SalesOrderNumber)
from FactOnlineSales

-- Related
select * from sys.indexes
where type IN (5,6)

select * from sys.dm_db_index_usage_stats order by object_id

-- index column info
select t.name as table_name,
       i.name as index_name,
	   col_name(ic.object_id, ic.column_id) as column_name,
	   ic.* 
from sys.tables t
join sys.indexes i
  on t.object_id = i.object_id	
join sys.index_columns ic
  on i.index_id = ic.index_id and t.object_id = ic.object_id
where i.type IN (5,6)

-- usage_stats
select t.name as table_name, 
       i.name as index_name, 
		us.*
from sys.tables t
join sys.indexes i
  on t.object_id = i.object_id	
join sys.dm_db_index_usage_stats us
  on us.object_id = i.object_id and us.index_id = i.index_id
where i.type IN (5,6)

-- dictionaries
SELECT	OBJECT_NAME(i.object_id) as TableName, 
		SUM(csd.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
FROM sys.indexes AS i
JOIN sys.partitions AS p
    ON i.object_id = p.object_id 
JOIN sys.column_store_dictionaries AS csd
    ON csd.hobt_id = p.hobt_id
where i.object_id = object_id('FactOnlineSales')
AND i.type_desc = 'CLUSTERED COLUMNSTORE'
group by OBJECT_NAME(i.object_id);
go

-- 230 rows - dictionary with name and partition info
SELECT	t.name, i.name, ic.column_id, col_name(ic.object_id, ic.column_id) as column_name, p.partition_number, 
		csd.dictionary_id,
		csd.type,
		csd.last_id,
		csd.entry_count,
		csd.on_disk_size
from sys.tables t
join sys.indexes i
  on t.object_id = i.object_id	
join sys.index_columns ic
  on i.index_id = ic.index_id and t.object_id = ic.object_id
JOIN sys.partitions AS p
    ON i.object_id = p.object_id 
JOIN sys.column_store_dictionaries AS csd
    ON csd.hobt_id = p.hobt_id AND csd.column_id = ic.column_id
where i.object_id = object_id('FactOnlineSales')
AND i.type_desc = 'CLUSTERED COLUMNSTORE'
order by ic.column_id
go

SELECT	OBJECT_NAME(i.object_id) as TableName, 
		csd.*
FROM sys.indexes AS i
JOIN sys.partitions AS p
    ON i.object_id = p.object_id 
JOIN sys.column_store_dictionaries AS csd
    ON csd.hobt_id = p.hobt_id
where i.object_id = object_id('FactOnlineSales')
AND i.type_desc = 'CLUSTERED COLUMNSTORE'
go


-- Dictionaries
-- Analyze dictionaries
select object_name(object_id) table_name
	, dictionary_id
    , count(*) as 'Number of Dictionaries'
    , sum(entry_count) as 'Entry Count'
    , min(on_disk_size) as 'Min Size'
    , max(on_disk_size) as 'Max Size'
    , avg(on_disk_size) as 'Avg Size'
from sys.column_store_dictionaries dict
join sys.partitions part
    on dict.hobt_id = part.hobt_id --and dict.partition_id = dict.partition_id
group by object_id, dictionary_id 
order by object_name(object_id), dictionary_id 

-- Rowgroup sizes
select OBJECT_NAME(p.object_id) as TableName, p.partition_id, p.data_compression, p.data_compression_desc, rg.state, rg.state_description, 
	rg.total_rows as SegmentRows,  p.rows as PartitionRows, rg.size_in_bytes as 'Size'
	from sys.partitions p
		inner join sys.column_store_row_groups rg
			on p.object_id = rg.object_id and p.partition_number = rg.partition_number 
	where p.object_id = object_id('dbo.FactOnlineSales')

-- rowgroup info, ordered
select *
from sys.column_store_row_groups rg
where rg.object_id = object_id('FactOnlineSales')
order by rg.partition_number, rg.row_group_id;

-- Enhanced rowgroup info
SELECT rg.total_rows, 
        cast(100.0*(total_rows)/1048576 as Decimal(6,3)) as PercentFull,
        cast(100-100.0*(total_rows - ISNULL(deleted_rows,0))/iif(total_rows = 0, 1, total_rows) as Decimal(6,3)) AS PercentDeleted, 
        i.object_id, object_name(i.object_id) AS TableName, 
        i.name AS IndexName, i.index_id, i.type_desc, 
        rg.*
FROM sys.indexes AS i
INNER JOIN sys.column_store_row_groups AS rg
    ON i.object_id = rg.object_id
AND i.index_id = rg.index_id 
WHERE object_name(i.object_id) = 'FactOnlineSales' 
--and state = 0
ORDER BY object_name(i.object_id), i.name, row_group_id;

-- Rowgroup counts and sizes
select	OBJECT_NAME(rg.object_id) as TableName, 
		count(*) as RowGroupsCount, 
		cast(sum(size_in_bytes) / 1024.0 / 1024  as Decimal(9,2)) as SizeInMb
from sys.column_store_row_groups rg
where object_id = object_id('FactOnlineSales')
group by OBJECT_NAME(rg.object_id)



-- Displaying the columns in the columnstore index
SELECT s.column_id, 
	col_name(ic.object_id, ic.column_id) as column_name,
    count(*) as segment_count
FROM sys.column_store_segments s 
join sys.partitions p
    ON s.partition_id = p.partition_id
LEFT JOIN sys.index_columns ic
	ON p.object_id = ic.object_id AND p.index_id = ic.index_id
		AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactOnlineSales'
GROUP BY s.column_id, col_name(ic.object_id, ic.column_id)
ORDER by s.column_id;
GO

-- Next is a query that provides the number of rows in each segment, 
-- but because each column has the same number of rows in each segment, 
-- the query is filtered on only a single column:
SELECT segment_id, sum(row_count) row_count
FROM sys.column_store_segments s join sys.partitions p
  ON s.partition_id = p.partition_id
JOIN sys.index_columns ic
  ON p.object_id = ic.object_id AND p.index_id = ic.index_id
  AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactOnlineSales' and index_column_id = 1
GROUP BY segment_id;
GO

-- Displaying information about each segment for one column
SELECT segment_id, min_data_id, max_data_id
FROM sys.column_store_segments s join sys.partitions p
   ON s.partition_id = p.partition_id
LEFT JOIN sys.index_columns ic
	ON p.object_id = ic.object_id AND p.index_id = ic.index_id
	AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactOnlineSales'
      AND col_name(ic.object_id, ic.column_id) = 'OrderDateKey'
ORDER by segment_id;
GO