
/*
Maintaining Batch Processing with Multiple Aggregates Including one or More DISTINCT Aggregates

Certain queries that use multiple aggregates, including at least one DISTINCT aggregate, may get less-than-optimal query plans with the columnstore index. They still probably will be no worse than older row-store plans, but because pure batch-mode columnstore plans are so fast, you may notice the difference more. Here's to higher expectations! 

All is not lost though. You can rewrite these queries to run fast by doing almost all their work in batch mode. For example, consider the following query:
*/
 
-- slow plan with spool
  
 select p.Date, 
count(distinct p.UserId) as UserIDCount, 
count(*) as CountAll 
from dbo.Purchase p, dbo.Media m 
where p.MediaId = m.MediaId 
and m.Category in ('Horror') 
group by p.Date 
 
The dbo.Purchase table has about 101 million rows, and a columnstore index on it. dbo.Media has a few thousand rows.  In SQL Server 2012 ("Denali") CTP3, this query generates a plan with a table spool. It saves millions of rows into an intermediate spool file with a single thread. Two separate branches of the query then read the spool to compute each of the aggregates separately.  This can take minutes on a 4-core processor. You can improve on this dramatically with the  following query rewrite. It computes each aggregate separately with a query block that contains only a single aggregate with grouping, and then  joins the two results together on the grouping key to form the final result.
 
-- workaround 

with A (Date, CountAll) as 
( 
select p.Date, 
count(*) as CountAll 
from dbo.Purchase p, dbo.Media m 
where p.MediaId = m.MediaId 
and m.Category in ('Horror') 
group by p.Date 
), 
B (Date, UserIdCount) as 
( 
select p.Date, 
count(distinct p.UserId) as UserIDCount 
from dbo.Purchase p, dbo.Media m 
where p.MediaId = m.MediaId 
and m.Category in ('Horror') 
group by p.Date 
) 
select A.Date, A.CountAll, B.UserIdCount 
from A join B on A.Date=B.Date 

The above query produces the same result as the first one but it runs in about a third of a second, compared to minutes for the original query.
 