
/*
INCREMENTAL = { ON | OFF } 
When ON, the statistics created are per partition statistics. When OFF, stats are combined for all partitions. The default is OFF. 

Affected syntax includes UPDATE STATISTICS, sp_createstats, CREATE INDEX, ALTER INDEX, 
ALTER DATABASE SET options, DATABASEPROPERTYEX, sys.databases, and sys.stats.

ALTER DATABASE SET AUTO_CREATE_STATISTICS { OFF | ON [ ( INCREMENTAL = { ON | OFF } ) ] } 
DATABASEPROPERTYEX, sys.databases, sys_stats (is incremental)

CREATE INDEX ... WITH STATISTICS_INCREMENTAL = { ON | OFF }
ALTER INDEX ... WITH STATISTICS_INCREMENTAL = { ON | OFF }

UPDATE STATISTICS ..  | RESAMPLE [ ON PARTITIONS ( { <partition_number> | <range> } [, n] ) ]
                      [ [ , ] INCREMENTAL = { ON | OFF } ]


*/

USE AdventureWorksDW2012;
GO

-- 43 partitions
CREATE PARTITION FUNCTION [ByOrderDateMonthPF](int) AS RANGE RIGHT 
FOR VALUES (
    20050701, 20050801, 20050901, 20051001, 20051101, 20051201, 
    20060101, 20060201, 20060301, 20060401, 20060501, 20060601, 
    20060701, 20060801, 20060901, 20061001, 20061101, 20061201, 
    20070101, 20070201, 20070301, 20070401, 20070501, 20070601, 
    20070701, 20070801, 20070901, 20071001, 20071101, 20071201, 
    20080101, 20080201, 20080301, 20080401, 20080501, 20080601, 
    20080701, 20080801, 20080901, 20081001, 20081101, 20081201
) 
GO

CREATE PARTITION SCHEME [ByOrderDateMonthRange] 
AS PARTITION [ByOrderDateMonthPF] 
ALL TO ([PRIMARY]) 
GO

-- Create a partitioned version of the FactResellerSales table
CREATE TABLE [dbo].[FactResellerSalesPtnd]( 
    [ProductKey] [int] NOT NULL, 
    [OrderDateKey] [int] NOT NULL, 
    [DueDateKey] [int] NOT NULL, 
    [ShipDateKey] [int] NOT NULL, 
    [ResellerKey] [int] NOT NULL, 
    [EmployeeKey] [int] NOT NULL, 
    [PromotionKey] [int] NOT NULL, 
    [CurrencyKey] [int] NOT NULL, 
    [SalesTerritoryKey] [int] NOT NULL, 
    [SalesOrderNumber] [nvarchar](20) NOT NULL, 
    [SalesOrderLineNumber] [tinyint] NOT NULL, 
    [RevisionNumber] [tinyint] NULL, 
    [OrderQuantity] [smallint] NULL, 
    [UnitPrice] [money] NULL, 
    [ExtendedAmount] [money] NULL, 
    [UnitPriceDiscountPct] [float] NULL, 
    [DiscountAmount] [float] NULL, 
    [ProductStandardCost] [money] NULL, 
    [TotalProductCost] [money] NULL, 
    [SalesAmount] [money] NULL, 
    [TaxAmt] [money] NULL, 
    [Freight] [money] NULL, 
    [CarrierTrackingNumber] [nvarchar](25) NULL, 
    [CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL,
) ON ByOrderDateMonthRange(OrderDateKey);
GO

-- Using simple or bulk logged recovery mode, and then the TABLOCK 
-- hint on the target table of the INSERTSELECT is a best practice
-- because it causes minimal logging and is therefore much faster.
ALTER DATABASE AdventureWorksDW2012 SET RECOVERY SIMPLE;
GO

-- Copy the data from the FactResellerSales into the new table
INSERT INTO dbo.FactResellerSalesPtnd WITH(TABLOCK)
SELECT * FROM dbo.FactResellerSales;
GO

-- Make it the default
ALTER DATABASE AdventureWorksDW2012 SET AUTO_CREATE_STATISTICS ON (INCREMENTAL = ON)
--No
--ALTER DATABASE AdventureWorksDW2012 SET AUTO_UPDATE_STATISTICS ON (INCREMENTAL = ON)
select * from sys.databases;

-- create incremental stats
-- An aggregate query joining two tables
-- Consider a star-join query that calculates how many of product 215 we sold each quarter. 
-- The fact table named FactResellerSalesPtnd is partitioned on the OrderDateKey column. 
SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPtnd AS f
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;
GO

-- info in sys.stats
select t.name as table_name, c.name as column_name, s.* 
from sys.stats s
join sys.stats_columns sc
on s.object_id = sc.object_id and s.stats_id = sc.stats_id 
join sys.tables t
on s.object_id = t.object_id
join sys.columns c
on sc.object_id = c.object_id and sc.column_id = c.column_id
where s.is_incremental = 1
GO

-- Make it not the default
ALTER DATABASE AdventureWorksDW2012 SET AUTO_CREATE_STATISTICS ON (INCREMENTAL = OFF)
GO

CREATE STATISTICS SalesOrderNumberStat
    ON dbo.FactResellerSalesPtnd (SalesOrderNumber)
	WITH INCREMENTAL = ON;

-- info in sys.stats
select t.name as table_name, c.name as column_name, s.* 
from sys.stats s
join sys.stats_columns sc
on s.object_id = sc.object_id and s.stats_id = sc.stats_id 
join sys.tables t
on s.object_id = t.object_id
join sys.columns c
on sc.object_id = c.object_id and sc.column_id = c.column_id
where s.is_incremental = 1
GO

select * 
from sys.partitions p
join sys.tables t
on p.object_id = t.object_id
where t.name = 'FactResellerSalesPtnd'

DBCC SHOW_STATISTICS ('FactResellerSalesPtnd', 'SalesOrderNumberStat')
GO

UPDATE STATISTICS FactResellerSalesPtnd (SalesOrderNumberStat)
	WITH RESAMPLE ON PARTITIONS (35,36,37),
	INCREMENTAL = ON;
GO

-- New statistics DMF. No information about partition here either
-- sys.dm_db_stats_properties Available 2014, 2012 SP1, 2008 R2 SP2
SELECT 
    sch.name + '.' + so.name AS "Table", 
    ss.name AS "Statistic", 
      CASE 
            WHEN ss.auto_Created = 0 AND ss.user_created = 0 THEN 'Index Statistic' 
            WHEN ss.auto_created = 0 AND ss.user_created = 1 THEN 'User Created' 
            WHEN ss.auto_created = 1 AND ss.user_created = 0 THEN 'Auto Created' 
            WHEN ss.auto_created = 1 AND ss.user_created = 1 THEN 'Not Possible?' 
      END AS "Statistic Type", 
    CASE 
            WHEN ss.has_filter = 1 THEN 'Filtered Index' 
            WHEN ss.has_filter = 0 THEN 'No Filter' 
      END AS "Filtered?", 
    CASE 
            WHEN ss.filter_definition IS NULL THEN '' 
            WHEN ss.filter_definition IS NOT NULL THEN ss.filter_definition 
      END AS "Filter Definition", 
    sp.last_updated AS "Stats Last Updated", 
    sp.rows AS "Rows", 
    sp.rows_sampled AS "Rows Sampled", 
    sp.unfiltered_rows AS "Unfiltered Rows", 
      sp.modification_counter AS "Row Modifications", 
      sp.steps AS "Histogram Steps", 
	sp.*
FROM sys.stats ss 
JOIN sys.objects so ON ss.object_id = so.object_id 
JOIN sys.schemas sch ON so.schema_id = sch.schema_id 
OUTER APPLY sys.dm_db_stats_properties(so.object_id, ss.stats_id) AS sp 
WHERE so.TYPE = 'U' 
AND sp.last_updated < getdate() - 30 
ORDER BY sp.last_updated DESC; 

-- single row
select * from sys.dm_db_stats_properties(OBJECT_ID('FactResellerSalesPtnd'), 4)




